const _ = require('underscore');
const axios = require('axios');

class Recipe {

  constructor() {
    // this.url = 'https://menu.silvrtree.co.uk/recipes';

    this.url = 'http://localhost:3000/recipes';
    this.listData = [];
    this.content = document.querySelector('#content');
    this.editor = document.querySelector('#editor');
    this.fields = ['_id', 'name', 'url', 'md', 'short', 'hash', 'lastused'];
    this.selects = ['meat', 'mealtype'];

    this.newButton = document.querySelector('#new');
    this.saveButton = document.querySelector('#save');
    this.deleteButton = document.querySelector('#delete');
    this.closeButton = document.querySelector('#close');

    /*
    <option value="1">Chicken</option>
                    <option value="2">Beef</option>
                    <option value="3">Pork</option>
                    <option value="4">Fish</option>
                    <option value="5">Egg</option>
                    <option value="6">Vegetable</option>

     */
    this.meatTypeTpl = [
      'nothing',
      '<span class=\'badge chicken\'>Chicken</span>',
      '<span class=\'badge beef\'>Beef</span>',
      '<span class=\'badge pork\'>Pork</span>',
      '<span class=\'badge fish\'>Fish</span>',
      '<span class=\'badge egg\'>Egg</span>',
      '<span class=\'badge vegetable\'>Vegetable</span>'

    ];

    this.mealTypeTpl = [
      'nothing',
      '',
      '<span class=\'badge badge-light\'>Soup</span>'

    ];
  }

  show(div) {
    div.style.display = '';
  }

  hide(div) {
    div.style.display = 'none';
  }

  resetForm() {
    this.fields.forEach((item) => {
      const elm = document.querySelector(`#${item}`);

      elm.value = '';
    });

    this.selects.forEach((item) => {
      const elm = document.querySelector(`#${item}`);

      elm.selectedIndex = 0;
    });
  }

  loadForm(workData) {
    this.fields.forEach((item) => {
      const elm = document.querySelector(`#${item}`);

      elm.value = workData[item];
    });

    this.selects.forEach((item) => {
      const elm = document.querySelector(`#${item}`);

      elm.selectedIndex = workData[item];
    });
  }

  getForm() {
    const newObj = {};
    this.fields.forEach((item) => {
      const elm = document.querySelector(`#${item}`);

      newObj[item] = elm.value;
    });

    this.selects.forEach((item) => {
      const elm = document.querySelector(`#${item}`);

      newObj[item] = elm.options[elm.selectedIndex].value;
    });

    return newObj;
  }

  setupClicks() {
    this.content.addEventListener('click', (event) => {
      try {
        const dataset = event.path[0].dataset;

        this.getRecord(dataset.hash.toString());
      }
      catch (e) {
        console.error(e);
      }
    });

    this.newButton.addEventListener('click', () => {
      setTimeout(() => {
        this.newForm();
      }, 100);
    });

    this.saveButton.addEventListener('click', () => {
      setTimeout(() => {
        this.sendRecord();
      }, 100);
    });

    this.closeButton.addEventListener('click', () => {
      setTimeout(() => {
        this.closeForm();
      }, 100);
    });

    this.deleteButton.addEventListener('click', () => {
      setTimeout(() => {
        this.deleteEntry();
      }, 300);
    });
  }

  deleteEntry() {
    if (confirm('Are you sure you want to delete this entry?')) {
      // Save it!
      const newData = this.getForm();

      this.deleteRecord(newData.hash);
    }
    else {
      // Do nothing!
    }
  }

  resetFormButtons() {
    const elm = document.querySelector('#hash');

    this.deleteButton.disabled = (elm.value === '');
  }

  newForm() {
    this.resetForm();

    this.resetFormButtons();
    this.show(this.editor);
    this.hide(this.content);
  }

  closeForm() {
    this.resetForm();

    this.resetFormButtons();
    this.hide(this.editor);
    this.show(this.content);
  }

  renderList() {
    const list = this.listData.map((item) => {
      console.log(item);

      return `<tr><td class="dataRow" data-id="${item._id}" data-hash="${item.hash}">${item.name}</td> <td>${this.mealTypeTpl[item.mealtype]} ${this.meatTypeTpl[item.meat]}</td><td><button class="btn btn-sm btn-primary" data-id="e_${item._id}" data-hash="${item.hash}" type="button">Edit</button></td></tr>`;
    });

    const tbody = list.join('');
    this.content.innerHTML = `<div class="cardV2"><table><tbody>${tbody}</tbody></table></div>`;
  }

  getList() {
    axios.get(this.url)
      .then( (response) => {
        // handle success

        this.listData = response.data.slice();
        this.renderList();
      })
      .catch( (error) => {
        // handle error
        console.error(error);
      });
  }

  getRecord(hash) {
    axios.get(`${this.url}/${hash}`)
      .then( (response) => {
        // handle success
        const data = Object.assign({}, response.data);

        this.resetForm();
        this.loadForm(data);
        this.resetFormButtons();
        this.show(this.editor);
        this.hide(this.content);
      })
      .catch( (error) => {
        // handle error
        console.error(error);
      });
  }

  postData(data) {
    axios.post(`${this.url}`, data)
      .then( (response) => {
        // handle success
        if (response.data._id > 0) {
          this.closeForm();

          setTimeout(() => {
            this.getList();
          }, 250);
        }
      })
      .catch( (error) => {
        // handle error
        console.error(error);
      });
  }

  updateData(data) {
    axios.put(`${this.url}/${data.hash}`, data)
      .then( (response) => {
        // handle success
        if (response.data.changes > 0) {
          this.closeForm();

          setTimeout(() => {
            this.getList();
          }, 250);
        }
      })
      .catch( (error) => {
        // handle error
        console.error(error);
      });
  }

  deleteRecord(hash) {
    axios.delete(`${this.url}/${hash}`)
      .then( (response) => {
        // handle success
        if (response.data.changes > 0) {
          this.closeForm();

          setTimeout(() => {
            this.getList();
          }, 250);
        }
      })
      .catch( (error) => {
        // handle error
        console.error(error);
      });
  }

  sendRecord() {
    const newData = this.getForm();

    if (newData.hash === '')
      this.postData(newData);
    else
      this.updateData(newData);
  }

  init() {
    console.log('init');

    this.resetForm();
    this.setupClicks();
    this.getList();
  }

}

(function () {
  const app = new Recipe();

  app.init();
})();

