const $ = require('jquery');
const _ = require('underscore');
const Backbone = require('backbone');

const tpl = require('./lib/utils');

const app = {

};

app.Recipe = Backbone.Model.extend({
  'urlRoot':'http://localhost:3000/recipes',
  'defaults':{
    '_id':null,
    'name':'',
    'hash':''
  }
});

app.RecipeCollection = Backbone.Collection.extend({
  'model':app.Recipe,
  'url':'http://localhost:3000/recipes'
});

app.HeaderView = Backbone.View.extend({

  'initialize':function () {
    console.log('>> HeaderView this', this);
    this.template = _.template(tpl.get('header'));
  },

  'render':function (eventName) {
    console.log('>> HeaderView render', this);
    $(this.el).html(this.template());

    return this;
  },

  'events':{
    'click .new':'newWine'
  },

  'newRecipe':function (event) {
    app.navigate('recipe/new', true);

    return false;
  }

});

app.RecipeListView = Backbone.View.extend({

  'tagName':'ul',

  'initialize':function () {
    this.model.bind('reset', this.render, this);
    var self = this;
    this.model.bind('add', function (recipe) {
      $(self.el).append(new app.RecipeListItemView({ 'model':recipe }).render().el);
    });
  },

  'render':function (eventName) {
    _.each(this.model.models, function (recipe) {
      $(this.el).append(new app.RecipeListItemView({ 'model':recipe }).render().el);
    }, this);

    return this;
  }
});

app.RecipeListItemView = Backbone.View.extend({

  'tagName':'li',

  'initialize':function () {
    this.template = _.template(tpl.get('recipe-list-item'));
    this.model.bind('change', this.render, this);
    this.model.bind('destroy', this.close, this);
  },

  'render':function (eventName) {
    $(this.el).html(this.template(this.model.toJSON()));

    return this;
  }

});

app.RecipeView = Backbone.View.extend({

  'tagName':'div', // Not required since 'div' is the default if no el or tagName specified

  'initialize':function () {
    this.template = _.template(tpl.get('recipe-details'));
    this.model.bind('change', this.render, this);
  },

  'render':function (eventName) {
    $(this.el).html(this.template(this.model.toJSON()));

    return this;
  },

  'events':{
    'change input':'change',
    'click .save':'saveRecipe',
    'click .delete':'deleteRecipe'
  },

  'change':function (event) {
    var target = event.target;
    console.log(`changing ${ target.id } from: ${ target.defaultValue } to: ${ target.value}`);
    // You could change your model on the spot, like this:
    // var change = {};
    // change[target.name] = target.value;
    // this.model.set(change);
  },

  'saveRecipe':function () {
    this.model.set({
      'name':$('#name').val()

    });
    if (this.model.isNew()) {
      var self = this;
      app.recipeList.create(this.model, {
        'success':function () {
          app.navigate(`recipe/${ self.model.id}`, false);
        }
      });
    }
    else
      this.model.save();

    return false;
  },

  'deleteRecipe':function () {
    this.model.destroy({
      'success':function () {
        alert('Recipe deleted successfully');
        window.history.back();
      }
    });

    return false;
  }

});

app.Router = Backbone.Router.extend({
  'initialize':function () {
    $('#header').html(new app.HeaderView().render().el);
  },
  'routes': {
    '': 'list',
    'recipe/new': 'newRecipe',
    'recipe/:id': 'recipeDetails'
  },

  'list':function () {
    this.before();
  },
  'recipeDetails':function (id) {
    this.before(function () {
      var recipe = app.router.recipeList.get(id);
      app.showView('#content', new app.RecipeView({ 'model':recipe }));
    });
  },

  'newRecipe':function () {
    this.before(function () {
      app.showView('#content', new app.RecipeView({ 'model':new app.recipe() }));
    });
  },

  'showView':function (selector, view) {
    if (this.currentView)
      this.currentView.close();
    $(selector).html(view.render().el);
    this.currentView = view;

    return view;
  },

  'before':function (callback) {
    console.log(this);
    if (this.recipeList) {
      if (callback) callback();
    }
    else {
      this.recipeList = new app.RecipeCollection();
      this.recipeList.fetch({ 'success': () => {
        console.log(this);
        console.log(app.router.recipeList);
        $('#sidebar').html(new app.RecipeListView({ 'model':app.router.recipeList }).render().el);
        if (callback) callback();
      } });
    }
  }

});

tpl.loadTemplates(['header', 'recipe-details', 'recipe-list-item'], function () {
  app.router = new app.Router();
  Backbone.history.start();
});
